#include "Ensemble.h"

Ensemble::Ensemble() :
    _tableau(0), _taille(0), _capacite(GRANULARITE_CAPACITE)
{
  _tableau = new int[_capacite];
}

Ensemble::Ensemble(unsigned int taille, int * tableau) :
    _tableau(0), _taille(taille), _capacite((taille / GRANULARITE_CAPACITE + 1) * GRANULARITE_CAPACITE)
{
  _tableau = new int[_capacite];
  for(unsigned int i=0; i<taille; i++) {
    _tableau[i] = tableau[i];
  }
}

Ensemble::Ensemble(const Ensemble & other) :
    _tableau(0), _taille(other._taille), _capacite((other._taille / GRANULARITE_CAPACITE + 1) * GRANULARITE_CAPACITE)
{
  this->_tableau = new int[_capacite];
  for (unsigned int i=0; i<other._taille; i++) {
    this->_tableau[i] = other._tableau[i];
  }
}

Ensemble & Ensemble::operator= (const Ensemble & other) {
  if (this != &other) {

    unsigned int capacite = (other._taille / GRANULARITE_CAPACITE + 1) * GRANULARITE_CAPACITE;

    this->ajusterCapacite(capacite);

    for (unsigned int i=0; i<other._taille; i++) {
      this->_tableau[i] = other._tableau[i];
    }
    this->_taille = other._taille;
  }
  return *this;
}

Ensemble::~Ensemble()
{
  delete[] _tableau;
}

bool Ensemble::estPresent(int e) const {
  bool present = false;

  for (unsigned int i=0; i<_taille; i++) {
    if (_tableau[i] == e) {
      present = true;
    }
  }

  return present;
}

void Ensemble::ajusterCapacite(unsigned int capacite) {

  if (this->_taille >= capacite) {
    capacite = (this->_taille / GRANULARITE_CAPACITE + 1) * GRANULARITE_CAPACITE;
  }

  if (this->_capacite != capacite) {
    int * tableau = new int[capacite];

    for (unsigned int i=0; i<this->_taille; i++) {
      tableau[i] = this->_tableau[i];
    }

    delete[] this->_tableau;
    this->_tableau = tableau;
    this->_capacite = capacite;
  }

}

bool Ensemble::ajouter(int e) {
  bool present = this->estPresent(e);

  if (!present) {

    if ( _taille == _capacite ) {
      ajusterCapacite(_capacite+GRANULARITE_CAPACITE);
    }

    _tableau[_taille] = e;
    _taille++;
  }

  return !present;
}

std::ostream & operator<<(std::ostream & os, const Ensemble & e) {
  os << "{ ";
  for (unsigned int i=0; i<e._taille; i++) {
    os << e._tableau[i] << " ";
  }
  os << "}";
  return os;
}
